// © Copyright The University of New South Wales 2018
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

#include "wrapper.h"
#include "max.h"

#include <stdio.h>

// Internal functions:
//-------------------------------------------------------------
// Starts an infinite listening thread
// to recieve ROS messages.
int launchthread(pthread_t *posixThreadID, char *err_msg);
// Stops the listening thread
// (blocks until stopped)
int killthread(pthread_t *posixThreadID, char *err_msg);
//-------------------------------------------------------------


int robin_start_listening(t_listener *listener, char *err_msg) {
    return launchthread(&(listener->posixThreadID), err_msg);
}

int robin_stop_listening(t_listener *listener, char *err_msg) {
    return killthread(&(listener->posixThreadID), err_msg);
}

int rout_open_twist(char rosSrvrIp[], char err_msg[]) {
    
    if( ros_init(rosSrvrIp) != 0) {
        sprintf(err_msg, "Error establishing a connection to the robot. Check your IP address.");
        return -1;
    }
    
    if( ros_advertise_twist() != 0) {
        sprintf(err_msg, "Error communicating with the robot.");
        return -1;
    }
    
    return 0;
}

int rout_open_chatter(char rosSrvrIp[], char err_msg[]) {
    
    ros_init(rosSrvrIp);
    ros_advertise_chatter();
    
    return 0;
}

int rout_close() {
    
    return 0;
}

int rout_set(char *key) {
    ros_publish_chatter(key);
    return 0;
}

int rout_set_twist(double out_msg[6]) {
    return ros_publish_twist(out_msg);
}


int isValidIpAddressCheck(char *ipAddress) {
    return isValidIpAddress(ipAddress);
}


// Starts an infinite listening thread to recieve ROS messages.
int launchthread(pthread_t *posixThreadID, char err_msg[]) //void *thread_args)
{
    // Create the thread using POSIX routines.
    pthread_attr_t  attr;
    int             returnVal;
    
    /*
    if(threadactive != 0) {
        sprintf(err_msg, "err thread is already running");
        //printf("err thread is already running\n");
        return -1;
    }
    */
    returnVal = pthread_attr_init(&attr);
    if(returnVal) {
        sprintf(err_msg, "err in pthread_attr_init");
        //printf("err in pthread_attr_init\n");
        return returnVal;
    }
    returnVal = pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    if(returnVal) {
        sprintf(err_msg, "err in pthread_attr_setdetachstate");
        //printf("err in pthread_attr_setdetachstate\n");
        return returnVal;
    }
    
    int threadError = pthread_create(posixThreadID, &attr, &ros_listen, NULL);
    if (threadError)
    {
        sprintf(err_msg, "err in pthread_create");
        //printf("err in pthread_create\n");
        return threadError;
    }
    
    //threadactive = 1;
    
    // Destroy the thread attributes object, no longer needed
    /*
    returnVal = pthread_attr_destroy(&attr);
    if (returnVal != 0)
    {
        sprintf(err_msg, "error in pthread_attr_destroy");
        //printf("error in pthread_attr_destroy\n");
        return -1;
    }*/
    
    return 0;
    
}

// Stops listening for incoming board messages (kill the listening thread).
int killthread(pthread_t *posixThreadID, char err_msg[])
{
    /*
    if(threadactive == 0) return 0;
    
    threadactive = 0;
    */
    // stop the running thread
    int returnVal = pthread_cancel(*posixThreadID);  // kill the listening thread without waiting
    //int returnVal = pthread_join(*posixThreadID, NULL); // BLOCK until thread finished
    if (returnVal != 0)
    {
        sprintf(err_msg, "error in pthread_cancel: %d", returnVal);
        //printf("error in killthread\n");
        return returnVal;
    }
    
    //printf("listening thread stopped\n");
    
    return 0;
}


