// © Copyright The University of New South Wales 2018
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

#ifndef wrapper_h
#define wrapper_h

#ifdef __cplusplus

#include "ros_lib/geometry_msgs/Twist.h"
#include "ros_lib/nav_msgs/Odometry.h"



//using namespace geometry_msgs;
const double* twistMsgToC(const geometry_msgs::Twist& received_msg);

extern "C" {
#endif
    
    // init ROS Node
    int ros_init(char *rosSrvrIp);
    
    // Publisher (output)
    void ros_advertise_chatter();
    int ros_advertise_twist();
    void ros_publish_chatter(const void *msg);
    int ros_publish_twist(const double out_msg[6]);
    
    // Subscriber (input)
    void ros_subscribe_twist(void *obj, void (*f)(void*, const void*, int), int index);
    void ros_subscribe_odometry(void *obj, void (*f)(void*, const void*, int), int index);
    void ros_subscribe_laser(void *obj, void (*f)(void*, const void*, int), int index);
    void ros_subscribe_imu(void *obj, void (*f)(void*, const void*, int), int index);
    void ros_subscribe_sensors(void *obj, void (*f)(void*, const void*, int), int index);
    void ros_subscribe_joints(void *obj, void (*f)(void*, const void*, int), int index);
    void ros_subscribe_tf(void *obj, void (*f)(void*, const void*, int), int index) ;
    
    void ros_listen(void *data);   
    
    int isValidIpAddress(char *ipAddress);
    
    
    double *twistMsg_floats(const void *msg);
    
    unsigned *laserMsg_header_ints(const void *msg);
    char *laserMsg_header_frame(const void *msg);
    double *laserMsg_floats(const void *msg);
    int laser_ranges_length(const void *msg);
    int laser_intensities_length(const void *msg);
    
    unsigned *odometryMsg_header_ints(const void *msg);
    char *odometryMsg_header_frame(const void *msg);
    char *odometryMsg_childframe(const void *msg);
    double *odometryMsg_floats(const void *msg);
    
    unsigned *imuMsg_header_ints(const void *msg);
    char *imuMsg_header_frame(const void *msg);
    double *imuMsg_floats(const void *msg);
    
    unsigned *jointsMsg_header_ints(const void *msg);
    char *jointsMsg_header_frame(const void *msg);
    int jointsMsg_length(const void *msg);
    char **jointsMsg_names(const void *msg);
    double *jointsMsg_floats(const void *msg);
    
    unsigned *sensorsMsg_ints(const void *msg);
    double sensorsMsg_float(const void *msg);
    
    unsigned *tfMsg_header_ints(const void *msg);
    char *tfMsg_header_frame(const void *msg);
    char *tfMsg_childframe(const void *msg);
    double *tfMsg_floats(const void *msg);


    typedef struct {
        char*       frame_id;
        unsigned    *header[2];//3
        char*       child_frame_id;
        double      *pose[2];//7
        double      *poseCovariance[2];//36
        double      *twist[2];    //2
        double      *twistCovariance[2];//36
    } msg_odometry;
    
    typedef struct {
        double      *twist[2];    //2
    } msg_velocity2;

    double twistMsg_get_linera_x(const void *msg);
    double twistMsg_get_linera_y(const void *msg);
    double twistMsg_get_linera_z(const void *msg);
    double twistMsg_get_angular_x(const void *msg);
    double twistMsg_get_angular_y(const void *msg);
    double twistMsg_get_angular_z(const void *msg);
    
    
#ifdef __cplusplus
}
#endif


#endif /* wrapper_h */
